import json

from aiohttp import web
from nacl.exceptions import BadSignatureError
from v6d0auth import certs

__all__ = ('V6D1TokensAppFactory',)

from v6d0auth.appfactory import AppFactory

from v6d1tokens.tdb import TDB


class V6D1TokensAppFactory(AppFactory):
    def __init__(self, tdb: TDB):
        self.tdb = tdb

    def define_routes(self, routes: web.RouteTableDef):
        print(certs.vkey.encode().hex())

        @routes.get('/')
        async def home(_request: web.Request):
            return web.Response(body='v6d1tokens\n')

        @routes.post('/reg')
        async def reg(request: web.Request):
            try:
                await self.tdb.reg(await request.read())
            except BadSignatureError:
                raise web.HTTPUnauthorized
            except json.JSONDecodeError:
                raise web.HTTPBadRequest
            else:
                raise web.HTTPOk

        @routes.post('/get')
        async def get(request: web.Request):
            try:
                token_encrypted = await self.tdb.get(await request.read())
            except BadSignatureError:
                raise web.HTTPUnauthorized
            except (json.JSONDecodeError, AssertionError):
                raise web.HTTPBadRequest
            except KeyError:
                raise web.HTTPNotFound
            else:
                return web.Response(body=token_encrypted)
