import json
from typing import Optional

from nacl.public import PublicKey, SealedBox
from nacl.signing import VerifyKey
from ptvp35 import Db, KVJson
from v6d0auth import certs

from v6d1tokens.config import myroot

__all__ = ('TDB',)

_tdbfile = myroot / 'tokens.db'


class TDB:
    def __init__(self):
        self.db = Db(_tdbfile, kvrequest_type=KVJson)

    def _get(self, token_id: str) -> Optional[str]:
        return self.db.get(token_id, None)

    async def get(self, request: bytes) -> bytes:
        token_id, requester_cert = json.loads(request)
        requester: PublicKey = VerifyKey(certs.averify(bytes.fromhex(requester_cert))).to_curve25519_public_key()
        token = self._get(token_id)
        if token is None:
            raise KeyError
        return SealedBox(requester).encrypt(token.encode())

    async def _reg(self, token_id: str, token: str) -> None:
        await self.db.set(token_id, token)

    async def reg(self, request: bytes) -> None:
        request = certs.verify(certs.receive(request))
        token_id, token = json.loads(request)
        assert type(token_id) == type(token) == str
        await self._reg(token_id, token)
